
#version 3.7;                   

global_settings
{
	assumed_gamma 1.0
}

#declare cameraLocation = <-5, 6, -18>;
#declare cameraLookAt = <0, 1.5, 0>;

camera {
	location cameraLocation
	up y*image_height right x*image_width
	look_at cameraLookAt
	angle 45
}

// ----------------------------------------

#macro PrintVector(v0)
    #debug concat("<", vstr(3, v0, ", ", 0, -1), "> \n")
#end

//  IsWithinRange_Value Macro by [GDS|Entropy] Ian McDonald 
#macro IsWithinRange_Value(Value, Min, Max)
    #local result = false;
    
    #if (Value < Max & Value > Min)
        #local result = true;
    #end
    
    result
#end

//  PrintOperations Macro by [GDS|Entropy] Ian McDonald
#macro PrintOperations(Value) 
    #debug concat("------------------------------ \n")
    #debug concat("[ Value ", str(Value, 20, 20), "] \n")
    #debug concat("[ exp ", str(exp(Value), 20, 20), "] \n")
    #debug concat("[ log ", str(log(Value), 20, 20), "] \n")
    #debug concat("[ ln ", str(ln(Value), 20, 20), "] \n")
    #debug concat("[ pow(log(Value), Value) ", str(pow(log(Value), Value), 20, 20), "] \n")
    #debug concat("[ pow(Value, 2) ", str(pow(Value, 2),0,0), "] \n")
    #debug concat("[ sin ", str(sin(Value), 20, 20), "] \n")
    #debug concat("[ sinh ", str(sinh(Value), 20, 20), "] \n")
    #debug concat("[ cos ", str(cos(Value), 20, 20), "] \n")
    #debug concat("[ cosh ", str(cosh(Value), 20, 20), "] \n")
    #debug concat("[ tan ", str(tan(Value), 20, 20), "] \n")
    #debug concat("[ tanh ", str(tanh(Value), 20, 200), "] \n")                                            
    #debug concat("[ sqrt ", str(sqrt(Value), 20, 20), "] \n")
    #debug concat("------------------------------ \n")
#end

//  Calculate Percent Macro by [GDS|Entropy] Ian Mcdonald
#macro CalculatePercent(Value, Total)
    ((Value / Total) * 100)
#end

// ----------------------------------------
 
#macro BlackHoleWarp(Iterations, Falloff, Strength, Position, Radius)
    #local counter = 0;
    #while (counter < Iterations) 
   
        warp 
        {
            black_hole Position, Radius
            falloff Falloff
            strength Strength
            turbulence 0.4
        }
   
        #local counter = counter + 1;
    #end
#end
		
#macro Glow(glowPosition, glowRadius, glowColor)
#local coreScaleFactor = 0.030625;   //0.00765625//0.0153125//0.030625 

#local lightFactor = (1 / 24);
#local density0 = true;
#local density1 = true;
#local density2 = true;

#local enableSurroundingFog = true;
#local enablePrimaryHalo = true;  // get "FixedSimpleVector internal limit exceeded" error when this = true and coreFuzzCount > 17
#local enableCoreFuzz = true;     // can also produce this error with various combinations of fuzzcount, primary halo enabled, and then rendering a selection
#local coreFuzzCount = 17;

#if (enableSurroundingFog)
//  Surrounding "Fog"
light_source 
{
    glowPosition       
    color rgb glowColor * lightFactor
    fade_distance 4 
    fade_power 2
    looks_like 
    { 
        sphere 
        { 
            <0, 0, 0>, glowRadius * 50
            hollow 
            pigment 
            { 
                rgbt <0, 0, 0, 1> 
            }
            interior 
            {
                media 
                {
                    intervals 2         
                    samples 10, 100          
                    emission glowColor
                    method 2
                    #if (density0) 
                        density
                        {
                            spherical
                            scale 8
                            density_map
                            {                         
                                [0.0 rgb glowColor * 0.0000]
                                [0.8 rgb glowColor * 0.00765625]
                            }                        
                        }
                    #end
                }
            }
            finish
            {
                reflection 0
                diffuse 0
                ambient 0
                phong 0
                specular 0
                brilliance 0                
            }             
        } 
    } 
}
#end

#if (enablePrimaryHalo)
//  Primary "Halo"
light_source 
{
    glowPosition       
    color rgb glowColor * lightFactor
    fade_distance 4 
    fade_power 2
    looks_like 
    { 
        sphere 
        { 
            <0, 0, 0>, glowRadius
            hollow 
            pigment 
            { 
                rgbt <0, 0, 0, 1> 
            }
            interior 
            {
                media 
                {
                    intervals 2         
                    samples 10, 100
                    emission glowColor
                    method 2
                    #if (density0) 
                        density
                        {
                            spherical
                            scale 3
                            density_map
                            {                         
                                [0.0 rgb glowColor * 0.000]
                                [0.8 rgb glowColor * 0.250]
                                [0.9 rgb glowColor * 0.750]
                                [1.0 rgb glowColor * 1.000]                        
                            }                        
                        }
                    #end
                    #if (density1)
                        density
                        { 
                            spherical
                            density_map
                            {
                                [0.00 color rgb glowColor * 0.000]
                                [ 0.90
                                    granite
                                    color_map 
                                    {
                                        [0 color rgb glowColor * 0.750]
                                        [1 color rgb glowColor * 0.250]
                                    }
                                    BlackHoleWarp(4, 1, 1, <0,0,0>, 1)
                                ]
                                [1.00 color rgb glowColor * 1.500]                        
                            }                                                                      
                        } 
                    #end 
                    #if (density2)
                        density
                        { 
                            spherical
                            density_map
                            {
                                [0.00 color rgb glowColor * 0.000]
                                [ 0.90
                                    granite
                                    color_map 
                                    {
                                        [0 color rgb glowColor * 2.750]
                                        [1 color rgb glowColor * 2.250]
                                    }
                                ]
                                [1.00 color rgb glowColor * 2.500]                        
                            }                                                                      
                        } 
                    #end                                       
                }
            }
            finish
            {
                reflection 0
                diffuse 0
                ambient 0
                phong 0
                specular 0
                brilliance 0                
            }             
        } 
    } 
}
#end

#if (enableCoreFuzz)
//  Core "Fuzz"
#local j = 1;
    light_source 
    {
        glowPosition       
        color rgb (glowColor * lightFactor) *2
        fade_distance 4 
        fade_power 2
        looks_like 
        {       
            sphere 
            { 
                <0, 0, 0>, ((glowRadius * 0.25) / (j * 0.05))
                hollow 
                pigment 
                { 
                    rgbt <0, 0, 0, 1> 
                }
                interior 
                {
                    media 
                    {
                        intervals 2         
                        samples 10, 200          
                        emission glowColor
                        method 2
                        #if (density0) 
                            density
                            {
                                spherical
                                density_map
                                {
                                    [0.0 rgb glowColor * 0.0000] 
                                     
                                    #local j = 1;
                                    #local jMax = 255;
                                    #while (j < jMax)
                                   
                                        #local percentOfTotal = CalculatePercent(j, jMax);

                                        #local r3 = 35;     //  30
                                        #local r2 = 30;     //  20
                                        #local r1 = 25;     //  10
                                        
                                        #local d1 = 1;      //   5
                                        #local d2 = 75;     //   50
                                        #local d3 = 100;    //  100
                                        #local d4 = 200;    //  200
                                        #local d5 = 300;    //  300
                                        
                                        #if (IsWithinRange_Value(percentOfTotal, r3, 100))
                                            #local factor = (j / d1) + ln(j);
                                            //#local factor = (j / ln(j));   
                                            //#local factor = percentOfTotal / d1;                                            
                                        #elseif (IsWithinRange_Value(percentOfTotal, r2, r3))
                                            #local factor = (j / d2) + log(j);                                             
                                            //#local factor = (j / log(j));
                                            //#local factor = ln(percentOfTotal) / d2;                                            
                                        #elseif (IsWithinRange_Value(percentOfTotal, r1, r2))
                                            #local factor = (j / d3); 
                                            //#local factor = ln(percentOfTotal) / d3;                                                      
                                        #elseif (IsWithinRange_Value(percentOfTotal, 0, r1))
                                            #local factor = (j / d4);
                                            //#local factor = percentOfTotal / d4;                                             
                                        #else
                                            #local factor = (j / d5);
                                            //#local factor = percentOfTotal / d5;
                                        #end
                                        
                                        //PrintOperations(j)
                                        
                                        //  Glow Type 1
                                        //[j * 0.01 rgb glowColor * (factor)]
                                        
                                        //  Glow type 3
                                        [(((j / jMax) * 100) * 0.1) rgb glowColor * (factor)]
                                    #local j = j + 1;
                                    #end
                                }                        
                            }
                        #end
                    }
                }
                finish
                {
                    reflection 0
                    diffuse 0
                    ambient 0
                    phong 0
                    specular 0
                    brilliance 0                
                }             
            } 
        } 
    }
#end

#end   
// ---------------------------------------- 
  
   
//Types:
#local Float = 0;
#local Vector = < 0, 0, 0>;
#local Color = color rgb < 0, 0, 0>;

#macro SRand(RS) (rand(RS)*2 - 1) #end
#macro RRand(RS, Min, Max) (rand(RS)*(Max-Min) + Min) #end
#macro Clamp(V, Min, Max) (min(Max, max(Min, V))) #end
#macro Range(V, Rmn, Rmx) (V*(Rmx-Rmn) + Rmn) #end
#macro RClamp(V, Rmn, Rmx, Min, Max) (Clamp(Range(V, Rmn, Rmx), Min, Max)) #end

#local RsA = seed(574647);

#local J = Float;
#local J = 0;
#while(J<52)
    Glow(< SRand(RsA)*8, rand(RsA)*4, SRand(RsA)*8>, rand(RsA)*1.5, 0.65*< RRand(RsA, 0.1, 1), RRand(RsA, 0.1, 1), RRand(RsA, 0.1, 1)>)
	#local J = J + 1;
#end

box
{
	<-15, 0,-10>,
	< 15, 0, 10>
	pigment
	{
	    checker
	    color rgb 0.05,
	    color rgb 1
	}
	finish
	{
		reflection
		{
		    0.5, 0.75 metallic
		}
		diffuse 0.25
		ambient 0
	}
}

sphere
{
	< 0, 2, 0>, 2
	pigment
	{
	    rgb 1
	}
	finish
	{
		reflection
		{
		    0.5, 0.85
		    metallic
		}
		diffuse 0.15
		ambient 0
	}
}
